// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CORE_CLIENTCACHE_H
#define CORE_CLIENTCACHE_H

#include <map>
#include <set>

#include <QObject>
#include <QString>

#include <core/storage/cache.h>
#include <shared/clientinfo.h>
#include <shared/identity.h>

namespace Core {

class ClientCache : public QObject {
    Q_OBJECT
public:
    ClientCache();
    ~ClientCache();

    void open();
    void close();

signals:
    void requestClientInfo(const QString& id);

public slots:
    bool checkClient(const QString& node, const QString& ver, const QString& hash);
    bool registerClientInfo(const QString& sourceFullJid, const QString& id, const std::set<Shared::Identity>& identities, const std::set<QString>& features);

private:
    std::map<QString, Shared::ClientInfo> requested;
    Cache<QString, Shared::ClientInfo> cache;
    std::map<QString, Shared::ClientInfo> specific;
};

}


#endif // CORE_CLIENTCACHE_H
